/*
 * This file is a part of Netacka 3D.
 *
 * Copyright 2010
 * Leszek Bartkowski, Mikołaj Dądela, Paweł Marczewski, Jan Szejko
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


package netacka3d;

import java.util.List;

import netacka3d.data.*;

import org.junit.*;

import static org.junit.Assert.*;

public class PlayersTableTest {

    private PlayersTable pT = new PlayersTable();
    private Player p1 = new Player(1, "aaa", PlayerColor.BLUE);
    private Player p2 = new Player(10, "bbb", PlayerColor.RED);
    private Player p3 = new Player(3, "ccc", PlayerColor.GRAY);
    private Player p4 = new Player(8, "ddd", PlayerColor.YELLOW);
    
    @Before
    public void playersTableTest() {
        Init.init();
        pT.addPlayer(p1);
        pT.addPlayer(p2);
        pT.addPlayer(p3);
        pT.addPlayer(p4);
    }
    
    @Test(expected=AssertionError.class)
    public void testAdd() {
        pT.addPlayer(pT.getPlayer(8));
    }
    
    @Test
    public void testRemove() {
        assertEquals(p4, pT.getPlayer(8));
        pT.removePlayer(8);
        assertNull(pT.getPlayer(8));
    }
    
    @Test
    public void testGet() {
        assertEquals(p2, pT.getPlayer(PlayerColor.RED));
        assertNull(pT.getPlayer(PlayerColor.PINK));
        assertEquals(p3, pT.getPlayer("ccc"));
        assertNull(pT.getPlayer("halo"));
        assertNull(pT.getPlayer(14365));
    }
    
    @Test
    public void testResolve () {
        Player p = pT.resolveConflicts(p2);
        assertNotNull(p);
        assertTrue(!p.nickname.equals("bbb"));
    }
    
    @After
    public void finish() {
        pT.checkConsistency();
    }
    
}
