/*
 * This file is a part of Netacka 3D.
 *
 * Copyright 2010
 * Leszek Bartkowski, Mikołaj Dądela, Paweł Marczewski, Jan Szejko
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


package netacka3d.graphics.pc; //#.pc
//#.android package netacka3d.graphics.android;

/* 
 * Kod OpenGLowy C++-owej Netacki 3D:
 * http://code.assembla.com/netacka3d-old/subversion/nodes/iface_opengl.cpp?rev=22
 */

import java.util.*;
//import java.nio.ByteBuffer;
//import java.nio.ByteOrder;
//import java.nio.FloatBuffer;

//import javax.vecmath.*;

import javax.media.opengl.*;            //#.pc
import javax.media.opengl.glu.GLU;      //#.pc

//#.android import javax.microedition.khronos.egl.EGLConfig;
//#.android import javax.microedition.khronos.opengles.GL10;

//#.android import android.opengl.GLU;
//#.android import android.opengl.GLSurfaceView.Renderer;
//#.android import android.util.Log;

import netacka3d.logic.*;
import netacka3d.misc.Logger;
import netacka3d.data.*;
import netacka3d.data.arena.Mesh;
import netacka3d.data.arena.MeshData;

//#replace GL GL10
//#replace glClearDepth glClearDepthf
//#replace glFogi glFogx
//#replace glLightModeli glLightModelx

public class GameRenderer
    implements GLEventListener //#.pc
    //#.android implements Renderer
{   
    private Listener gameListener;
    //private ArenaRenderer arenaRenderer;
    private MeshRenderer arenaRenderer;
    private TextRenderer textRenderer;
    private TreeMap<Integer, RibbonHandler> ribbons;
    private RibbonHandler myRibbon;
    private int myId;
    
    private long stepStartTime;
    private int stepNum;
    private MatchSettings matchSettings;
    private Game.MatchState matchState;
    private boolean stateChanged;
    private PlayersTable players;
    private boolean running;
 
    // nowa arena, jeśli trzeba zmienić
    private MeshData newArena;
    
    private boolean showStats;
    // numer aktywnego gracza którego śledzimy
    private int toSpectateNum;
    
    private int width, height;    
    
    private GLU glu; //#.pc
    
    public GameRenderer(Controller con) {
        glu = new GLU(); //#.android
        
        gameListener = new Listener();
        //arenaRenderer = new ArenaRenderer();
        textRenderer = new TextRenderer();
        ribbons = new TreeMap<Integer, RibbonHandler>();
        
        // zaczynamy z domyślnymi ustawieniami
        matchSettings = new MatchSettings();
        matchState = Game.MatchState.MATCH_WARMUP;        
        myId = 0;
        running = false;        
        stepNum = 0;
        stepStartTime = System.currentTimeMillis();
        showStats = false;
        toSpectateNum = -1;
        
        con.registerGameListener(gameListener);
    }
    
    
    /// Zdarzenia OpenGL
    
    public void init(GLAutoDrawable dr) //#.pc 
    //#.android public synchronized void onSurfaceCreated(GL10 gl, EGLConfig config)
    {
        GL gl = dr.getGL(); //#.pc        
        
        /* Ustawienia OpenGL */

        gl.glHint(GL.GL_PERSPECTIVE_CORRECTION_HINT, GL.GL_NICEST);

        gl.glShadeModel(GL.GL_SMOOTH);

        /* tło */
        gl.glClearColor(0.0f, 0.0f, 0.0f, 0.0f);

        gl.glClearDepth(1.0f); // !
        gl.glEnable(GL.GL_DEPTH_TEST);
        gl.glDepthFunc(GL.GL_LEQUAL);

        /* oświetlenie - od białego do szarego */
        
        gl.glEnable(GL.GL_LIGHT0);

        gl.glEnable(GL.GL_LIGHTING);

        gl.glLightModeli(GL.GL_LIGHT_MODEL_TWO_SIDE, GL.GL_TRUE);
        
        float[] ambientColor = new float[] { 1.0f, 1.0f, 1.0f, 1.0f };
        float[] diffuseColor = new float[] { 1.0f, 1.0f, 1.0f, 1.0f };

        gl.glLightfv(GL.GL_LIGHT0, GL.GL_AMBIENT, ambientColor, 0);
        gl.glLightfv(GL.GL_LIGHT0, GL.GL_DIFFUSE, diffuseColor, 0);
        
        gl.glLineWidth(2.0f);

        gl.glEnableClientState(GL.GL_VERTEX_ARRAY);
        
        gl.glFrontFace(GL.GL_CCW);
        
        textRenderer.init(gl); 
    }

    public synchronized void destroy(GL gl) {
        textRenderer.destroy(gl);
        if (arenaRenderer != null)
            arenaRenderer.destroy(gl);
    }
    
    public synchronized void reshape(GLAutoDrawable dr, int x, int y, int width, //#.pc
            int height) //#.pc
    //#.android public synchronized void onSurfaceChanged(GL10 gl, int width, int height)
    {
        GL gl = dr.getGL(); //#.pc
        
        this.width = width;
        this.height = height;
        textRenderer.setScreenDimensions(width, height);
        
        resetSettings(gl);
    }
    
    public void display(GLAutoDrawable dr) //#.pc 
    //#.android public synchronized void onDrawFrame(GL10 gl)
    {
        GL gl = dr.getGL(); //#.pc
        
        if (newArena != null) {
            if (arenaRenderer != null)
                arenaRenderer.destroy(gl);
            arenaRenderer = new MeshRenderer(newArena);
            newArena = null;
        }
        
    synchronized(this) {
        if (stateChanged) {
            stateChanged = false;
            resetSettings(gl);
        }
        
        /* Czyścimy ekran */
        gl.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT);
        gl.glMatrixMode(GL.GL_MODELVIEW);
        gl.glLoadIdentity();
        
        /* Ustawiamy światło */
        float[] lightPosition = new float[] { 0.0f, -1.0f, 0.0f, 1.0f };
        gl.glLightfv(GL.GL_LIGHT0, GL.GL_POSITION, lightPosition, 0);
                
        // bieżąca chwila czasu, użyta do ustawienia kamery
        RoundTime rt;
        
        // id gracza ktorego śledzimy
        int toFollowId = -1;
        // czy jesteśmy w spectator mode
        boolean spectating = false;
        
        switch (matchState) {
        case ROUND:
            rt = getRoundTime();
            if (myRibbon != null) {
                RoundTime death = players.getPlayer(myId).death;
                // kamera zostaje na graczu przez ~1 sekundę po jego śmierci
                if (death != null && rt.t - death.t > matchSettings.stepRate) {
                    spectating = true;
                }
            } else
                spectating = true;
            
            if (spectating) {
                List<Player> ps = players.getActivePlayers();
                if (toSpectateNum > ps.size())
                    toSpectateNum = -1;
                if (toSpectateNum != -1)
                    toFollowId = ps.get(toSpectateNum).id;
                else
                    toFollowId = -1;
            } else
                    toFollowId = myId;
            break;
        case ROUND_WARMUP:
            // pozycje startowe
            rt = new RoundTime(0, 0);
            toFollowId = myId;
            break;
        default:
            // w ogóle nie ustawiamy kamery na graczu
            rt = null;
            break;
        }

        boolean cull = false;
        RibbonHandler rhFollow = ribbons.get(toFollowId);
        if (!(rhFollow != null && rhFollow.calcCameraPosition(gl, rt))) {
            // pozycja dla spectator mode
            gl.glTranslatef(0, 0, -matchSettings.arenaSize);
            cull = true;
        }
                
        gl.glPushMatrix();
        gl.glEnable(GL.GL_RESCALE_NORMAL);
        gl.glScalef(matchSettings.arenaSize,
                matchSettings.arenaSize,
                matchSettings.arenaSize);
        if (cull) {
            gl.glEnable(GL.GL_CULL_FACE);
            gl.glCullFace(GL.GL_FRONT);
        }
        if (arenaRenderer != null)
            arenaRenderer.display(gl);
        if (cull)
            gl.glDisable(GL.GL_CULL_FACE);
        gl.glDisable(GL.GL_RESCALE_NORMAL);
        gl.glPopMatrix();
        
        for (RibbonHandler rh: ribbons.values())
            rh.display(gl, rt);
        
        textRenderer.startFrame(gl);        
        //displayDebugInfo(gl);
        
        if (showStats) {
            displayStats(gl);        
        } else {            
            switch(matchState) {
            case MATCH_WARMUP:
                textRenderer.setColor(gl, 1, 1, 1);
                textRenderer.drawStringCenter(gl, 3, "Waiting for players");
                break;
            case ROUND_WARMUP:
                textRenderer.setColor(gl, 1, 1, 1);
                textRenderer.drawStringCenter(gl, 3, "Get Ready");
                break;
            case ROUND:                
                if (spectating) {
                    textRenderer.setColor(gl, 1, 1, 1);
                    if (toFollowId == -1) 
                        textRenderer.drawStringCenter(gl, 2, "Spectating");
                    else {
                        textRenderer.drawStringCenter(gl, 2, "Following");
                        Player p = players.getPlayer(toFollowId);
                        PlayerColor c = p.color;
                        textRenderer.setColor(gl, c.r, c.g, c.b);
                        textRenderer.drawStringCenter(gl, 3, p.nickname);
                        textRenderer.setColor(gl, 1, 1, 1);
                    }
                    textRenderer.drawStringCenter(gl, 4, 
                            "Press Space to change" //#.pc
                            //#.android "Tap screen to change"
                            );
                }
                break;
            case ROUND_FINISHED:
                displayStats(gl);
                break;
            case MATCH_FINISHED:
                textRenderer.setColor(gl, 1, 0.5f, 0.5f);
                textRenderer.drawStringCenter(gl, 3, "Game Over");
                displayStats(gl);
                break;
            }
        }
        
        textRenderer.endFrame(gl);
        
        gl.glFlush();
    } // synchronized(this)
    }
    
    private RoundTime getRoundTime() {
        long now = System.currentTimeMillis();
        float d = stepNum + (now - stepStartTime) * matchSettings.stepRate / 1000;
        if (d < 0)
            d = 0;
        return new RoundTime((int)d, d-(int)d);
    }
    
    // niepotrzebny w wersji Androidowej
    public synchronized void displayChanged(GLAutoDrawable dr, //#.pc
            boolean modeChanged, boolean deviceChanged) //#.pc
    {} //#.pc
    
    
    /// Zmień ustawienia zależne od MatchSettings
    private void resetSettings(GL gl) {
        // Mgła
        
        float fogColor[] = { 0.0f, 0.0f, 0.0f, 1.0f };

        gl.glFogi(GL.GL_FOG_MODE, GL.GL_LINEAR);
        gl.glFogf(GL.GL_FOG_DENSITY, 0.2f);
        gl.glFogf(GL.GL_FOG_START, matchSettings.arenaSize/2);
        gl.glFogf(GL.GL_FOG_END, matchSettings.arenaSize*1.7f);
        gl.glFogfv(GL.GL_FOG_COLOR, fogColor, 0);

        gl.glEnable(GL.GL_FOG);
        
        // Viewport
        
        gl.glViewport(0, 0, width, height);

        gl.glMatrixMode(GL.GL_PROJECTION);
        gl.glLoadIdentity();

        glu.gluPerspective( //#.pc
        //#.android GLU.gluPerspective(gl,
                /* FOV degrees */
                100.0f,
                /* aspect ratio */
                (float)width/(float)height,
                /* near clip */
                0.2f*matchSettings.segmentWidth,
                /* far clip */
                matchSettings.arenaSize*200);

        gl.glMatrixMode(GL.GL_MODELVIEW);
        gl.glLoadIdentity();
    }
    
    public synchronized void setShowStats(boolean showStats) {
        this.showStats = showStats;
    }
    
    public synchronized void spectateNext() {
        ++toSpectateNum;
        if (players == null || 
                toSpectateNum >= players.getActivePlayers().size())
            toSpectateNum = -1;
    }
    
    private void displayDebugInfo(GL gl) {
        RoundTime t = getRoundTime();
        
        String diff = "";

        if (t.t < stepNum)
            diff = " <";
        else if (t.t > stepNum)
            diff = " >";
        
        String[] ss = new String[] {
                "t-real = " + (t.t+t.tFrac),
                "t-serv = " + t.t + diff,
                matchState.toString()
        };
        
        textRenderer.setColor(gl, 1, 1, 1);
        for (int i = 0; i < ss.length; ++i)
           textRenderer.drawString(gl, 0, i, ss[i]);
    }
    
    private void displayStats(GL gl) {
        if (players == null) 
            return;
        List<Player> active = players.getActivePlayers();
        Collections.sort(active, new Comparator<Player>() {
            public int compare(Player p1, Player p2) {
                return p2.score - p1.score;
            }
        });
        
        for (int i = 0; i < active.size(); ++i) {
            Player p = active.get(i);
            textRenderer.setColor(gl, p.color.r, p.color.g, p.color.b);
            textRenderer.drawStringRight(gl, 4, 3+i, ""+p.score);
            if (!p.isAlive(stepNum))
                textRenderer.drawString(gl, 6, 3+i, "*");
            textRenderer.drawString(gl, 8, 3+i, ""+p.nickname);
        }
    }    
    
    /// Listener
    
    private class Listener implements GameListener {
        
        /* TODO czy tutaj mamy gwarancję że taki player został nam 
         * przedstawiony jako aktywny (= mamy go w ribbons)? */
        public void onSegment(int playerId, int t, Segment segment) {
//            Logger.i(String.format(
//                    "segment: id=%d t=%d %s", playerId, t, 
//                    segment));
            synchronized(GameRenderer.this) {
                ribbons.get(playerId).setSegment(t, segment);
            }
        }

        public void onStart(int playerId) {
            Logger.i("onStart("+playerId+")");
            synchronized(GameRenderer.this) {
                myId = playerId;
            }            
        }

        public void onStep(boolean stateChanged, Game game) {
            synchronized(GameRenderer.this) {
                long now = System.currentTimeMillis();
                if (!running)
                    stateChanged = true;
            
                if (stateChanged) {
                    GameRenderer.this.stateChanged = true;
                    stepStartTime = now;
                } else {
                    long projectedStart = stepStartTime + 
                        (long) ((game.stepNum - stepNum) * 
                            (1000/matchSettings.stepRate));
                    /* jeśli podany czas startu kroku nie różni się wiele od
                     * takiego, jaki nam wyszedł, używamy własnego.
                     */
                    long diff = projectedStart - now;
                    if (-80 < diff && diff < 80)
                        stepStartTime = projectedStart;
                    else
                        stepStartTime = now;
                }
                stepNum = game.stepNum;
                //Logger.i("Jest step "+stepNum);
                
                if (stateChanged) {
                    stepStartTime = now;
                    Logger.d("state: " + game.matchState.toString());   
                    if (!running || !game.matchSettings.arenaName.equals(
                                    matchSettings.arenaName))
                        newArena = game.matchSettings.arena;
                    matchSettings = game.matchSettings;
                    matchState = game.matchState;
                    //PlayersTable oldPlayers = players;
                    players = game.players.getCopy();
                    running = true;                    
                    /* TODO ustalić dokładnie co zmienia się przy jakich zmianach 
                     * stanu (czyli kiedy na przykład czyścić ribbons */
                    switch (matchState) {
                    case ROUND_WARMUP:                    
                        ribbons.clear();
                        break;
                    }
                    if (ribbons.isEmpty()) {
                        for (Player p: players.getActivePlayers())
                            ribbons.put(p.id, 
                                    new RibbonHandler(p.id, p.color, matchSettings));
                        myRibbon = ribbons.get(myId);
                    }
                    for (Player p: players.getActivePlayers()) {
                        if (p.death != null)
                            ribbons.get(p.id).endRibbon(p.death);                     
                    }
                }
            }
        }

        public void onStop(String reason) {
            synchronized(GameRenderer.this) {
                running = false;
            }
        }        
    }
}
