/*
 * This file is a part of Netacka 3D.
 *
 * Copyright 2010
 * Leszek Bartkowski, Mikołaj Dądela, Paweł Marczewski, Jan Szejko
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

/*
 * GamePanel.java
 *
 * Created on 2010-05-04, 12:42:42
 */

package netacka3d.ui.pc;

import java.awt.Dimension;
import java.awt.KeyEventDispatcher;
import java.awt.KeyboardFocusManager;
import java.awt.event.KeyEvent;
import java.awt.event.WindowEvent;
import java.awt.event.WindowFocusListener;

import javax.media.opengl.GLCapabilities;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;

import com.sun.opengl.util.Animator;
import com.sun.opengl.util.FPSAnimator;

import netacka3d.data.Direction;
import netacka3d.data.Segment;
import netacka3d.graphics.pc.GameRenderer;
import netacka3d.logic.*;
import netacka3d.misc.Logger;

/**
 *
 * @author janek
 */
public class GamePanel extends MainWindowPanel implements KeyEventDispatcher,
        WindowFocusListener {

    private GameRenderer renderer;
    private ClientController controller;
    private ServerController sCon;
    private Animator animator;
    private Listener listener;
    private boolean running;
    //private GLCanvas canvas;

    private int keyX, keyY, keyZ;
    private Direction lastDir;
    private boolean escapePressed = false;

    /** Creates new form GamePanel */
    public GamePanel(MainWindow window, ClientController controller,
            ServerController serverController) {
        super(window);
        initComponents();
        listener = new Listener();

        sCon = serverController;

        this.controller = controller;
        controller.unregisterGameListeners();
        controller.registerGameListener(listener);

        renderer = new GameRenderer(controller);

        // workaround - inaczej nie da się zmniejszać okna
        canvas.setMinimumSize(new Dimension(0,0));
        
        canvas.addGLEventListener(renderer);
        animator = new FPSAnimator(canvas, 60);

        KeyboardFocusManager.getCurrentKeyboardFocusManager().
            addKeyEventDispatcher(this);

        mainWindow.addWindowFocusListener(this);

        this.setFocusable(false);
        running = false;
    }

    public void startGame() {
        running = true;
        Logger.i("startGame");
        keyX = keyY = keyZ = 0;
        lastDir = Direction.STRAIGHT;
        updateKeys();
        animator.start();
    }

    public void stopGame(final String reason) {
        Logger.i("stopGame");
        if (sCon != null)
            sCon.endGame();
        if (running) {
            running = false;
            animator.stop();
            canvas.removeGLEventListener(renderer);
        }
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                if (running)
                    renderer.destroy(canvas.getGL());
                if (sCon != null)
                    changeTo(new NewGameMenu(mainWindow));
                else
                    changeTo(new JoinGameMenu(mainWindow));
                if (!escapePressed)
                    JOptionPane.showMessageDialog(mainWindow, reason);
            }
        });

    }

    public void updateKeys() {
        Direction dir = Direction.fromKeys(keyX, keyY, keyZ);
        if (dir.equals(lastDir))
            return;
        lastDir = dir;
        //Logger.i("new keys: " + dir.toString());
        if (!running)
            return;
        controller.setPlayerDirection(dir);
    }

    public void keyTyped(KeyEvent e) {
        //nic
    }

    @Override
    public void windowGainedFocus(WindowEvent e) {
        // nic
    }

    @Override
    public void windowLostFocus(WindowEvent e) {
        keyX = keyY = keyZ = 0;
        updateKeys();
    }

    private class Listener implements GameListener {
        public void onSegment(int playerId, int t, Segment segment) {
            //nic
        }

        public void onStart(int playerId) {
            startGame();
        }

        public void onStep(boolean stateChanged, Game game) {
            //nic
        }

        public void onStop(String reason) {
            stopGame(reason);
        }

    }

    @Override
    public boolean dispatchKeyEvent(KeyEvent e) {
        if (e.getID() == KeyEvent.KEY_PRESSED) {
            switch (Character.toLowerCase(e.getKeyChar())) {
            case 'a': keyY =  1; break;
            case 'd': keyY = -1; break;
            case 'w': keyX =  1; break;
            case 's': keyX = -1; break;
            case 'q': keyZ = -1; break;
            case 'e': keyZ =  1; break;
            case KeyEvent.VK_TAB:
                renderer.setShowStats(true);
                return true;
            case KeyEvent.VK_ESCAPE:
                controller.disconnect();
                escapePressed  = true;
                //this.changeTo(new MainMenu(mainWindow));
                return true;
            default:
                return false;
            }
            updateKeys();
            return true;
        } else if (e.getID() == KeyEvent.KEY_RELEASED) {
            switch (e.getKeyChar()) {
            case 'a': case 'd':
                keyY = 0;
                break;
            case 'w': case 's':
                keyX = 0;
                break;
            case 'q': case 'e':
                keyZ = 0;
                break;
            case '\t':
                renderer.setShowStats(false);
                return true;
            case ' ':
                renderer.spectateNext();
                return true;
            default:
                return false;
            }
            updateKeys();
            return true;
        }
        return false;
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        GLCapabilities cap = new GLCapabilities();
        cap.setDoubleBuffered(true);
        canvas = new javax.media.opengl.GLCanvas(cap);

        setMinimumSize(new java.awt.Dimension(640, 480));
        setPreferredSize(new java.awt.Dimension(640, 480));

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(canvas, javax.swing.GroupLayout.DEFAULT_SIZE, 640, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(canvas, javax.swing.GroupLayout.DEFAULT_SIZE, 480, Short.MAX_VALUE)
        );
    }// </editor-fold>//GEN-END:initComponents

    @Override
    public void onLeave() {
        KeyboardFocusManager.getCurrentKeyboardFocusManager().
            removeKeyEventDispatcher(this);
        mainWindow.removeWindowFocusListener(this);
    }


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.media.opengl.GLCanvas canvas;
    // End of variables declaration//GEN-END:variables

}
